##############################################################################
#
#    Context Methods, for Odoo
#    Copyright (C) 2021, 2022, 2023 XCG Consulting <https://xcg-consulting.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from odoo.models import BaseModel


class Base(BaseModel):
    """Add :py:meth:~odoo.models.Model.get_clean_context and
    :py:meth:~odoo.models.Model.with_lang method on models."""

    _inherit = "base"

    def get_clean_context(self) -> dict[str, str]:
        """Remove cruft from the Odoo context.

        Useful when returning view display actions.
        """

        return {
            key: value
            for key, value in self.env.context.items()
            if (
                not key.startswith("default_")
                and not key.startswith("search_default_")
                and key not in ("form_view_ref", "group_by", "tree_view_ref")
            )
        }

    def with_lang(self) -> BaseModel:
        """Add lang info from the current user regardless of what the context may
        already contain. It so happens the context often has wrong lang info by
        default.

        Useful when running automated tasks.

        :return: That same Odoo record set with added lang info.
        """

        return self.with_context(lang=self.env.user.lang)
